"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Shard = exports.ShardKind = void 0;
const tslib_1 = require("tslib");
/* eslint-disable no-dupe-class-members */
const ipfs_core_1 = require("ipfs-core");
const buffer_1 = require("buffer");
const errors_1 = require("./errors");
const events_1 = require("events");
const _1 = require(".");
var ShardKind;
(function (ShardKind) {
    // eslint-disable-next-line no-unused-vars
    ShardKind["Binary"] = "binary";
    // eslint-disable-next-line no-unused-vars
    ShardKind["Object"] = "object";
})(ShardKind = exports.ShardKind || (exports.ShardKind = {}));
const toUint8Array = (buf) => {
    const a = new Uint8Array(buf.length);
    for (let i = 0; i < buf.length; i++)
        a[i] = buf[i];
    return a;
};
const createCID = (data, kind, ipfs) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    if (kind === ShardKind.Binary) {
        const file = yield ipfs.add(toUint8Array(data));
        return file.cid;
    }
    if (kind === ShardKind.Object) {
        return ipfs.dag.put(data);
    }
    throw new errors_1.UnknownShardKindError();
});
class Shard {
    constructor(stack, cid, kind, data) {
        this.stack = stack;
        this.cid = cid;
        this.kind = kind;
        this.data = data;
        this.events = new events_1.EventEmitter();
    }
    put(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.cid = yield createCID(data, this.kind, this.stack.ipfs);
            this.events.emit('update', this);
        });
    }
    on(event, listener) {
        this.events.on(event, listener);
    }
    static new(stack, kind, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            return new Shard(stack, yield createCID(data, kind, stack.ipfs), kind, data);
        });
    }
    static create(stack, cid, kind) {
        var e_1, _a;
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let data;
            if (kind === ShardKind.Binary) {
                data = buffer_1.Buffer.from('');
                try {
                    for (var _b = tslib_1.__asyncValues(stack.ipfs.cat(cid)), _c; _c = yield _b.next(), !_c.done;) {
                        const chunk = _c.value;
                        data = buffer_1.Buffer.concat([data, buffer_1.Buffer.from(chunk)]);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) yield _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            else if (kind === ShardKind.Object) {
                data = (yield stack.ipfs.dag.get(cid)).value;
            }
            return new Shard(stack, cid, kind, data);
        });
    }
    static from(stack, path) {
        // eslint-disable-next-line no-unused-vars
        const [_, shard, cid, kind] = path.split('/');
        if (shard !== 'shard') {
            throw new _1.InvalidShardPathError();
        }
        try {
            return Shard.create(stack, ipfs_core_1.CID.parse(cid), kind);
        }
        catch (error) {
            throw new _1.InvalidShardPathError(error);
        }
    }
    toString() {
        return `/shard/${this.cid.toString()}/${this.kind}`;
    }
}
exports.Shard = Shard;
//# sourceMappingURL=shard.js.map