"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Store = void 0;
const tslib_1 = require("tslib");
const shard_1 = require("./shard");
class Store {
    constructor(stack, storage) {
        this.stack = stack;
        this.pubsub = stack.pubsub.create('$$store');
        this.storage = storage;
    }
    onReplicate(msg) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (msg.data.kind !== 'replicate')
                return;
            const local = yield this.storage.get(msg.data.key);
            const date = new Date(msg.data.date);
            if ((local === null || local === void 0 ? void 0 : local.date) && (local === null || local === void 0 ? void 0 : local.date) > date)
                return;
            yield this.storage.set(msg.data.key, {
                value: msg.data.value,
                date
            });
        });
    }
    watchShard(key, watch) {
        watch.on('update', (shard) => tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.storage.set(key, {
                value: shard.toString(),
                date: new Date()
            });
            yield this.replicate(key);
        }));
    }
    get(key) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const local = yield this.storage.get(key);
            if (!local)
                return null;
            const shard = yield shard_1.Shard.from(this.stack, local.value);
            this.watchShard(key, shard);
            return shard;
        });
    }
    set(key, shard) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.storage.set(key, {
                value: shard.toString(),
                date: new Date()
            });
            this.watchShard(key, shard);
            yield this.replicate(key);
        });
    }
    replicate(key) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const local = yield this.storage.get(key);
            if (!local)
                return;
            yield this.pubsub.publish('replicate', {
                kind: 'replicate',
                date: local.date.toISOString(),
                key,
                value: local.value
            });
        });
    }
    start() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.pubsub.subscribe('replicate', (msg) => this.onReplicate(msg));
            yield this.pubsub.subscribe('replicateRequest', (msg) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                if (msg.data.kind !== 'replicateRequest')
                    return;
                const keys = yield this.storage.keys();
                yield Promise.all(keys.map(this.replicate));
            }));
            yield this.pubsub.publish('replicateRequest', { kind: 'replicateRequest' });
        });
    }
    stop() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.pubsub.stop();
        });
    }
}
exports.Store = Store;
//# sourceMappingURL=store.js.map